@@ -6,7 +6,7 @@
  *
  * Copyright (C) 2005 David Zeuthen, <davidz@redhat.com>
  * Copyright (C) 2005 - 2008 Dan Williams, <dcbw@redhat.com>
- * Copyright (C) 2005 - 2010 Red Hat, Inc.
+ * Copyright (C) 2005 - 2011 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,7 +35,6 @@
 #include <glib/gi18n-lib.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 #include <gnome-keyring.h>
 #include <gnome-keyring-memory.h>
 
@@ -84,10 +83,11 @@ G_DEFINE_TYPE_EXTENDED (VpncPluginUiWidget, vpnc_plugin_ui_widget, G_TYPE_OBJECT
 #define VPNC_PLUGIN_UI_WIDGET_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VPNC_TYPE_PLUGIN_UI_WIDGET, VpncPluginUiWidgetPrivate))
 
 typedef struct {
-	GladeXML *xml;
+	GtkBuilder *builder;
 	GtkWidget *widget;
 	GtkSizeGroup *group;
 	gint orig_dpd_timeout;
+	gboolean new_connection;
 } VpncPluginUiWidgetPrivate;
 
 
@@ -137,7 +137,7 @@ check_validity (VpncPluginUiWidget *self, GError **error)
 	GtkWidget *widget;
 	char *str;
 
-	widget = glade_xml_get_widget (priv->xml, "gateway_entry");
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "gateway_entry"));
 	str = (char *) gtk_entry_get_text (GTK_ENTRY (widget));
 	if (!str || !strlen (str) || strstr (str, " ") || strstr (str, "\t")) {
 		g_set_error (error,
@@ -147,7 +147,7 @@ check_validity (VpncPluginUiWidget *self, GError **error)
 		return FALSE;
 	}
 
-	widget = glade_xml_get_widget (priv->xml, "group_entry");
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "group_entry"));
 	str = (char *) gtk_entry_get_text (GTK_ENTRY (widget));
 	if (!str || !strlen (str)) {
 		g_set_error (error,
@@ -175,48 +175,44 @@ fill_vpn_passwords (VpncPluginUiWidget *self, NMConnection *connection)
 	char *password = NULL;
 	char *group_password = NULL;
 
-	/* Grab secrets from the keyring */
+	/* Grab secrets from the connection or the keyring */
 	if (connection) {
 		NMSettingConnection *s_con;
 		NMSettingVPN *s_vpn;
+		NMSettingSecretFlags secret_flags = NM_SETTING_SECRET_FLAG_NONE;
 		const char *tmp;
 
+		s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION));
+
 		s_vpn = (NMSettingVPN *) nm_connection_get_setting (connection, NM_TYPE_SETTING_VPN);
+		if (s_vpn) {
+			tmp = nm_setting_vpn_get_secret (s_vpn, NM_VPNC_KEY_XAUTH_PASSWORD);
+			if (tmp)
+				password = gnome_keyring_memory_strdup (tmp);
 
-		if (nm_connection_get_scope (connection) == NM_CONNECTION_SCOPE_SYSTEM) {
-			if (s_vpn) {
-				tmp = nm_setting_vpn_get_secret (s_vpn, NM_VPNC_KEY_XAUTH_PASSWORD);
-				if (tmp)
-					password = gnome_keyring_memory_strdup (tmp);
+			tmp = nm_setting_vpn_get_secret (s_vpn, NM_VPNC_KEY_SECRET);
+			if (tmp)
+				group_password = gnome_keyring_memory_strdup (tmp);
+		}
 
-				tmp = nm_setting_vpn_get_secret (s_vpn, NM_VPNC_KEY_SECRET);
-				if (tmp)
-					group_password = gnome_keyring_memory_strdup (tmp);
-			}
-		} else {
-			s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION));
-
-			/* Lookup passwords in the keyring, and if they weren't there, try
-			 * the connection itself, which is where they'd be right after import.
-			 */
+		nm_setting_get_secret_flags (NM_SETTING (s_vpn), NM_VPNC_KEY_XAUTH_PASSWORD, &secret_flags, NULL);
+		if (!password && (secret_flags & NM_SETTING_SECRET_FLAG_AGENT_OWNED)) {
 			keyring_helpers_get_one_secret (nm_setting_connection_get_uuid (s_con),
-			                                VPNC_USER_PASSWORD,
-			                                &password,
-			                                NULL);
-			if (!password)
-				password = gnome_keyring_memory_strdup (nm_setting_vpn_get_secret (s_vpn, NM_VPNC_KEY_XAUTH_PASSWORD));
+				                            VPNC_USER_PASSWORD,
+				                            &password);
+		}
 
+		secret_flags = NM_SETTING_SECRET_FLAG_NONE;
+		nm_setting_get_secret_flags (NM_SETTING (s_vpn), NM_VPNC_KEY_SECRET, &secret_flags, NULL);
+		if (!group_password && (secret_flags & NM_SETTING_SECRET_FLAG_AGENT_OWNED)) {
 			keyring_helpers_get_one_secret (nm_setting_connection_get_uuid (s_con),
 			                                VPNC_GROUP_PASSWORD,
-			                                &group_password,
-			                                NULL);
-			if (!group_password)
-				group_password = gnome_keyring_memory_strdup (nm_setting_vpn_get_secret (s_vpn, NM_VPNC_KEY_SECRET));
+			                                &group_password);
 		}
 	}
 
 	/* User password */
-	widget = glade_xml_get_widget (priv->xml, "user_password_entry");
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "user_password_entry"));
 	if (!widget)
 		goto out;
 	if (password)
@@ -225,7 +221,7 @@ fill_vpn_passwords (VpncPluginUiWidget *self, NMConnection *connection)
 	g_signal_connect (G_OBJECT (widget), "changed", G_CALLBACK (stuff_changed_cb), self);
 
 	/* Group password */
-	widget = glade_xml_get_widget (priv->xml, "group_password_entry");
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "group_password_entry"));
 	if (!widget)
 		goto out;
 	if (group_password)
@@ -251,11 +247,11 @@ show_toggled_cb (GtkCheckButton *button, VpncPluginUiWidget *self)
 
 	visible = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button));
 
-	widget = glade_xml_get_widget (priv->xml, "user_password_entry");
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "user_password_entry"));
 	g_assert (widget);
 	gtk_entry_set_visibility (GTK_ENTRY (widget), visible);
 
-	widget = glade_xml_get_widget (priv->xml, "group_password_entry");
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "group_password_entry"));
 	g_assert (widget);
 	gtk_entry_set_visibility (GTK_ENTRY (widget), visible);
 }
@@ -266,26 +262,24 @@ pw_type_changed_helper (VpncPluginUiWidget *self, GtkWidget *combo)
 	VpncPluginUiWidgetPrivate *priv = VPNC_PLUGIN_UI_WIDGET_GET_PRIVATE (self);
 	const char *entry = NULL;
 	GtkWidget *widget;
-	GtkTreeModel *model;
 
 	/* If the user chose "Not required", desensitize and clear the correct
 	 * password entry.
 	 */
-	widget = glade_xml_get_widget (priv->xml, "user_pass_type_combo");
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "user_pass_type_combo"));
 	if (combo == widget)
 		entry = "user_password_entry";
 	else {
-		widget = glade_xml_get_widget (priv->xml, "group_pass_type_combo");
+		widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "group_pass_type_combo"));
 		if (combo == widget)
 			entry = "group_password_entry";
 	}
 	if (!entry)
 		return;
 
-	widget = glade_xml_get_widget (priv->xml, entry);
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, entry));
 	g_assert (widget);
 
-	model = gtk_combo_box_get_model (GTK_COMBO_BOX (combo));
 	switch (gtk_combo_box_get_active (GTK_COMBO_BOX (combo))) {
 	case PW_TYPE_ASK:
 	case PW_TYPE_UNUSED:
@@ -307,11 +301,27 @@ pw_type_combo_changed_cb (GtkWidget *combo, gpointer user_data)
 	stuff_changed_cb (combo, self);
 }
 
+static const char *
+secret_flags_to_pw_type (NMSettingVPN *s_vpn, const char *key)
+{
+	NMSettingSecretFlags flags = NM_SETTING_SECRET_FLAG_NONE;
+
+	if (nm_setting_get_secret_flags (NM_SETTING (s_vpn), key, &flags, NULL)) {
+		if (flags & NM_SETTING_SECRET_FLAG_NOT_REQUIRED)
+			return NM_VPNC_PW_TYPE_UNUSED;
+		if (flags & NM_SETTING_SECRET_FLAG_NOT_SAVED)
+			return NM_VPNC_PW_TYPE_ASK;
+		return NM_VPNC_PW_TYPE_SAVE;
+	}
+	return NULL;
+}
+
 static void
 init_one_pw_combo (VpncPluginUiWidget *self,
                    NMSettingVPN *s_vpn,
                    const char *combo_name,
-                   const char *key,
+                   const char *secret_key,
+                   const char *type_key,
                    const char *entry_name)
 {
 	VpncPluginUiWidgetPrivate *priv = VPNC_PLUGIN_UI_WIDGET_GET_PRIVATE (self);
@@ -325,7 +335,7 @@ init_one_pw_combo (VpncPluginUiWidget *self,
 	/* If there's already a password and the password type can't be found in
 	 * the VPN settings, default to saving it.  Otherwise, always ask for it.
 	 */
-	widget = glade_xml_get_widget (priv->xml, entry_name);
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, entry_name));
 	if (widget) {
 		const char *tmp;
 
@@ -335,8 +345,11 @@ init_one_pw_combo (VpncPluginUiWidget *self,
 	}
 
 	store = gtk_list_store_new (1, G_TYPE_STRING);
-	if (s_vpn)
-		value = nm_setting_vpn_get_data_item (s_vpn, key);
+	if (s_vpn) {
+		value = secret_flags_to_pw_type (s_vpn, secret_key);
+		if (!value)
+			value = nm_setting_vpn_get_data_item (s_vpn, type_key);
+	}
 
 	gtk_list_store_append (store, &iter);
 	gtk_list_store_set (store, &iter, 0, _("Saved"), -1);
@@ -359,7 +372,7 @@ init_one_pw_combo (VpncPluginUiWidget *self,
 			active = 2;
 	}
 
-	widget = glade_xml_get_widget (priv->xml, combo_name);
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, combo_name));
 	g_assert (widget);
 	gtk_combo_box_set_model (GTK_COMBO_BOX (widget), GTK_TREE_MODEL (store));
 	g_object_unref (store);
@@ -380,12 +393,13 @@ init_plugin_ui (VpncPluginUiWidget *self, NMConnection *connection, GError **err
 	const char *value = NULL;
 	int active = -1;
 	const char *natt_mode = NULL;
+	const char *ike_dh_group = NULL;
 
 	s_vpn = (NMSettingVPN *) nm_connection_get_setting (connection, NM_TYPE_SETTING_VPN);
 
 	priv->group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 
-	widget = glade_xml_get_widget (priv->xml, "gateway_entry");
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "gateway_entry"));
 	g_return_val_if_fail (widget != NULL, FALSE);
 	gtk_size_group_add_widget (priv->group, GTK_WIDGET (widget));
 	if (s_vpn) {
@@ -395,7 +409,7 @@ init_plugin_ui (VpncPluginUiWidget *self, NMConnection *connection, GError **err
 	}
 	g_signal_connect (G_OBJECT (widget), "changed", G_CALLBACK (stuff_changed_cb), self);
 
-	widget = glade_xml_get_widget (priv->xml, "group_entry");
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "group_entry"));
 	g_return_val_if_fail (widget != NULL, FALSE);
 	gtk_size_group_add_widget (priv->group, GTK_WIDGET (widget));
 	if (s_vpn) {
@@ -405,7 +419,7 @@ init_plugin_ui (VpncPluginUiWidget *self, NMConnection *connection, GError **err
 	}
 	g_signal_connect (G_OBJECT (widget), "changed", G_CALLBACK (stuff_changed_cb), self);
 
-	widget = glade_xml_get_widget (priv->xml, "encryption_combo");
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "encryption_combo"));
 	g_return_val_if_fail (widget != NULL, FALSE);
 	gtk_size_group_add_widget (priv->group, GTK_WIDGET (widget));
 
@@ -440,12 +454,20 @@ init_plugin_ui (VpncPluginUiWidget *self, NMConnection *connection, GError **err
 	 */
 	fill_vpn_passwords (self, connection);
 
-	init_one_pw_combo (self, s_vpn, "user_pass_type_combo",
-	                   NM_VPNC_KEY_XAUTH_PASSWORD_TYPE, "user_password_entry");
-	init_one_pw_combo (self, s_vpn, "group_pass_type_combo",
-	                   NM_VPNC_KEY_SECRET_TYPE, "group_password_entry");
+	init_one_pw_combo (self,
+	                   s_vpn,
+	                   "user_pass_type_combo",
+	                   NM_VPNC_KEY_XAUTH_PASSWORD,
+	                   NM_VPNC_KEY_XAUTH_PASSWORD_TYPE,
+	                   "user_password_entry");
+	init_one_pw_combo (self,
+	                   s_vpn,
+	                   "group_pass_type_combo",
+	                   NM_VPNC_KEY_SECRET,
+	                   NM_VPNC_KEY_SECRET_TYPE,
+	                   "group_password_entry");
 
-	widget = glade_xml_get_widget (priv->xml, "user_entry");
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "user_entry"));
 	g_return_val_if_fail (widget != NULL, FALSE);
 	gtk_size_group_add_widget (priv->group, GTK_WIDGET (widget));
 	if (s_vpn) {
@@ -455,7 +477,7 @@ init_plugin_ui (VpncPluginUiWidget *self, NMConnection *connection, GError **err
 	}
 	g_signal_connect (G_OBJECT (widget), "changed", G_CALLBACK (stuff_changed_cb), self);
 
-	widget = glade_xml_get_widget (priv->xml, "domain_entry");
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "domain_entry"));
 	g_return_val_if_fail (widget != NULL, FALSE);
 	gtk_size_group_add_widget (priv->group, GTK_WIDGET (widget));
 	if (s_vpn) {
@@ -498,7 +520,7 @@ init_plugin_ui (VpncPluginUiWidget *self, NMConnection *connection, GError **err
 			active = 3;
 	}
 
-	widget = glade_xml_get_widget (priv->xml, "natt_combo");
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "natt_combo"));
 	g_return_val_if_fail (widget != NULL, FALSE);
 	gtk_size_group_add_widget (priv->group, GTK_WIDGET (widget));
 	gtk_combo_box_set_model (GTK_COMBO_BOX (widget), GTK_TREE_MODEL (store));
@@ -506,7 +528,41 @@ init_plugin_ui (VpncPluginUiWidget *self, NMConnection *connection, GError **err
 	gtk_combo_box_set_active (GTK_COMBO_BOX (widget), active < 0 ? 0 : active);
 	g_signal_connect (G_OBJECT (widget), "changed", G_CALLBACK (stuff_changed_cb), self);
 
-	widget = glade_xml_get_widget (priv->xml, "disable_dpd_checkbutton");
+	active = -1;
+	store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
+	if (s_vpn)
+		ike_dh_group = nm_setting_vpn_get_data_item (s_vpn, NM_VPNC_KEY_DHGROUP);
+
+	gtk_list_store_append (store, &iter);
+	gtk_list_store_set (store, &iter, 0, _("DH Group 1"), 1, NM_VPNC_DHGROUP_DH1, -1);
+	if ((active < 0) && ike_dh_group) {
+		if (!strcmp (ike_dh_group, NM_VPNC_DHGROUP_DH1))
+			active = 0;
+	}
+
+	gtk_list_store_append (store, &iter);
+	gtk_list_store_set (store, &iter, 0, _("DH Group 2 (default)"), 1, NM_VPNC_DHGROUP_DH2, -1);
+	if ((active < 0) && ike_dh_group) {
+		if (!strcmp (ike_dh_group, NM_VPNC_DHGROUP_DH2))
+			active = 1;
+	}
+
+	gtk_list_store_append (store, &iter);
+	gtk_list_store_set (store, &iter, 0, _("DH Group 5"), 1, NM_VPNC_DHGROUP_DH5, -1);
+	if ((active < 0) && ike_dh_group) {
+		if (!strcmp (ike_dh_group, NM_VPNC_DHGROUP_DH5))
+			active = 2;
+	}
+
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "dhgroup_combo"));
+	g_return_val_if_fail (widget != NULL, FALSE);
+	gtk_size_group_add_widget (priv->group, GTK_WIDGET (widget));
+	gtk_combo_box_set_model (GTK_COMBO_BOX (widget), GTK_TREE_MODEL (store));
+	g_object_unref (store);
+	gtk_combo_box_set_active (GTK_COMBO_BOX (widget), active < 0 ? 1 : active);
+	g_signal_connect (G_OBJECT (widget), "changed", G_CALLBACK (stuff_changed_cb), self);
+
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "disable_dpd_checkbutton"));
 	g_return_val_if_fail (widget != NULL, FALSE);
 	if (s_vpn) {
 		value = nm_setting_vpn_get_data_item (s_vpn, NM_VPNC_KEY_DPD_IDLE_TIMEOUT);
@@ -524,7 +580,7 @@ init_plugin_ui (VpncPluginUiWidget *self, NMConnection *connection, GError **err
 	}
 	g_signal_connect (G_OBJECT (widget), "toggled", G_CALLBACK (stuff_changed_cb), self);
 
-	widget = glade_xml_get_widget (priv->xml, "show_passwords_checkbutton");
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "show_passwords_checkbutton"));
 	g_return_val_if_fail (widget != NULL, FALSE);
 	g_signal_connect (G_OBJECT (widget), "toggled",
 	                  (GCallback) show_toggled_cb,
@@ -543,30 +599,46 @@ get_widget (NMVpnPluginUiWidgetInterface *iface)
 }
 
 static guint32
-handle_one_pw_type (NMSettingVPN *s_vpn, GladeXML *xml, const char *name, const char *key)
+handle_one_pw_type (NMSettingVPN *s_vpn,
+                    GtkBuilder *builder,
+                    const char *combo_name,
+                    const char *secret_key,
+                    const char *type_key,
+                    gboolean new_connection)
 {
+	NMSettingSecretFlags flags = NM_SETTING_SECRET_FLAG_NONE;
 	GtkWidget *widget;
-	GtkTreeModel *model;
 	guint32 pw_type;
+	const char *data_val = NULL;
 
-	widget = glade_xml_get_widget (xml, name);
-	model = gtk_combo_box_get_model (GTK_COMBO_BOX (widget));
+	nm_setting_get_secret_flags (NM_SETTING (s_vpn), secret_key, &flags, NULL);
+	flags &= ~(NM_SETTING_SECRET_FLAG_NOT_SAVED | NM_SETTING_SECRET_FLAG_NOT_REQUIRED);
 
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, combo_name));
 	pw_type = gtk_combo_box_get_active (GTK_COMBO_BOX (widget));
 	switch (pw_type) {
 	case PW_TYPE_SAVE:
-		nm_setting_vpn_add_data_item (s_vpn, key, NM_VPNC_PW_TYPE_SAVE);
+		data_val = NM_VPNC_PW_TYPE_SAVE;
 		break;
 	case PW_TYPE_UNUSED:
-		nm_setting_vpn_add_data_item (s_vpn, key, NM_VPNC_PW_TYPE_UNUSED);
+		data_val = NM_VPNC_PW_TYPE_UNUSED;
+		flags |= NM_SETTING_SECRET_FLAG_NOT_REQUIRED;
 		break;
 	case PW_TYPE_ASK:
 	default:
 		pw_type = PW_TYPE_ASK;
-		nm_setting_vpn_add_data_item (s_vpn, key, NM_VPNC_PW_TYPE_ASK);
+		data_val = NM_VPNC_PW_TYPE_ASK;
+		flags |= NM_SETTING_SECRET_FLAG_NOT_SAVED;
 		break;
 	}
 
+	if (new_connection) {
+		/* new connections default to agent-owned secrets */
+		flags |= NM_SETTING_SECRET_FLAG_AGENT_OWNED;
+	}
+
+	nm_setting_vpn_add_data_item (s_vpn, type_key, data_val);
+	nm_setting_set_secret_flags (NM_SETTING (s_vpn), secret_key, flags, NULL);
 	return pw_type;
 }
 
@@ -591,28 +663,28 @@ update_connection (NMVpnPluginUiWidgetInterface *iface,
 	g_object_set (s_vpn, NM_SETTING_VPN_SERVICE_TYPE, NM_DBUS_SERVICE_VPNC, NULL);
 
 	/* Gateway */
-	widget = glade_xml_get_widget (priv->xml, "gateway_entry");
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "gateway_entry"));
 	str = (char *) gtk_entry_get_text (GTK_ENTRY (widget));
 	if (str && strlen (str))
 		nm_setting_vpn_add_data_item (s_vpn, NM_VPNC_KEY_GATEWAY, str);
 
 	/* Group name */
-	widget = glade_xml_get_widget (priv->xml, "group_entry");
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "group_entry"));
 	str = (char *) gtk_entry_get_text (GTK_ENTRY (widget));
 	if (str && strlen (str))
 		nm_setting_vpn_add_data_item (s_vpn, NM_VPNC_KEY_ID, str);
 
-	widget = glade_xml_get_widget (priv->xml, "user_entry");
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "user_entry"));
 	str = (char *) gtk_entry_get_text (GTK_ENTRY (widget));
 	if (str && strlen (str))
 		nm_setting_vpn_add_data_item (s_vpn, NM_VPNC_KEY_XAUTH_USER, str);
 
-	widget = glade_xml_get_widget (priv->xml, "domain_entry");
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "domain_entry"));
 	str = (char *) gtk_entry_get_text (GTK_ENTRY (widget));
 	if (str && strlen (str))
 		nm_setting_vpn_add_data_item (s_vpn, NM_VPNC_KEY_DOMAIN, str);
 
-	widget = glade_xml_get_widget (priv->xml, "encryption_combo");
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "encryption_combo"));
 	switch (gtk_combo_box_get_active (GTK_COMBO_BOX (widget))) {
 	case ENC_TYPE_WEAK:
 		nm_setting_vpn_add_data_item (s_vpn, NM_VPNC_KEY_SINGLE_DES, "yes");
@@ -625,7 +697,7 @@ update_connection (NMVpnPluginUiWidgetInterface *iface,
 		break;
 	}
 
-	widget = glade_xml_get_widget (priv->xml, "natt_combo");
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "natt_combo"));
 	model = gtk_combo_box_get_model (GTK_COMBO_BOX (widget));
 	if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (widget), &iter)) {
 		const char *mode = NULL;
@@ -635,7 +707,17 @@ update_connection (NMVpnPluginUiWidgetInterface *iface,
 	} else
 		nm_setting_vpn_add_data_item (s_vpn, NM_VPNC_KEY_NAT_TRAVERSAL_MODE, NM_VPNC_NATT_MODE_NATT);
 
-	widget = glade_xml_get_widget (priv->xml, "disable_dpd_checkbutton");
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "dhgroup_combo"));
+	model = gtk_combo_box_get_model (GTK_COMBO_BOX (widget));
+	if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (widget), &iter)) {
+		const char *dhgroup = NULL;
+
+		gtk_tree_model_get (model, &iter, 1, &dhgroup, -1);
+		nm_setting_vpn_add_data_item (s_vpn, NM_VPNC_KEY_DHGROUP, dhgroup);
+	} else
+		nm_setting_vpn_add_data_item (s_vpn, NM_VPNC_KEY_DHGROUP, NM_VPNC_DHGROUP_DH2);
+
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "disable_dpd_checkbutton"));
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) {
 		nm_setting_vpn_add_data_item (s_vpn, NM_VPNC_KEY_DPD_IDLE_TIMEOUT, "0");
 	} else {
@@ -650,25 +732,30 @@ update_connection (NMVpnPluginUiWidgetInterface *iface,
 		}
 	}
 
-	upw_type = handle_one_pw_type (s_vpn, priv->xml, "user_pass_type_combo", NM_VPNC_KEY_XAUTH_PASSWORD_TYPE);
-	gpw_type = handle_one_pw_type (s_vpn, priv->xml, "group_pass_type_combo", NM_VPNC_KEY_SECRET_TYPE);
+	upw_type = handle_one_pw_type (s_vpn,
+	                               priv->builder,
+	                               "user_pass_type_combo",
+	                               NM_VPNC_KEY_XAUTH_PASSWORD,
+	                               NM_VPNC_KEY_XAUTH_PASSWORD_TYPE,
+	                               priv->new_connection);
+	gpw_type = handle_one_pw_type (s_vpn,
+	                               priv->builder,
+	                               "group_pass_type_combo",
+	                               NM_VPNC_KEY_SECRET,
+	                               NM_VPNC_KEY_SECRET_TYPE,
+	                               priv->new_connection);
 
-	/* System secrets get stored in the connection, user secrets are saved
-	 * via the save_secrets() hook.
-	 */
-	if (nm_connection_get_scope (connection) == NM_CONNECTION_SCOPE_SYSTEM) {
-		/* User password */
-		widget = glade_xml_get_widget (priv->xml, "user_password_entry");
-		str = (char *) gtk_entry_get_text (GTK_ENTRY (widget));
-		if (str && strlen (str) && (upw_type != PW_TYPE_UNUSED))
-			nm_setting_vpn_add_secret (s_vpn, NM_VPNC_KEY_XAUTH_PASSWORD, str);
+	/* User password */
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "user_password_entry"));
+	str = (char *) gtk_entry_get_text (GTK_ENTRY (widget));
+	if (str && strlen (str) && (upw_type != PW_TYPE_UNUSED))
+		nm_setting_vpn_add_secret (s_vpn, NM_VPNC_KEY_XAUTH_PASSWORD, str);
 
-		/* Group password */
-		widget = glade_xml_get_widget (priv->xml, "group_password_entry");
-		str = (char *) gtk_entry_get_text (GTK_ENTRY (widget));
-		if (str && strlen (str) && (gpw_type != PW_TYPE_UNUSED))
-			nm_setting_vpn_add_secret (s_vpn, NM_VPNC_KEY_SECRET, str);
-	}
+	/* Group password */
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "group_password_entry"));
+	str = (char *) gtk_entry_get_text (GTK_ENTRY (widget));
+	if (str && strlen (str) && (gpw_type != PW_TYPE_UNUSED))
+		nm_setting_vpn_add_secret (s_vpn, NM_VPNC_KEY_SECRET, str);
 
 	/* Local Port is not in GUI (yet?). So when present in the connection,
 	 * copy it from the old VPN setting to the new one to preserve it.
@@ -685,7 +772,7 @@ update_connection (NMVpnPluginUiWidgetInterface *iface,
 }
 
 static void
-save_one_password (GladeXML *xml,
+save_one_password (GtkBuilder *builder,
                    const char *keyring_tag,
                    const char *uuid,
                    const char *id,
@@ -699,11 +786,11 @@ save_one_password (GladeXML *xml,
 	GtkTreeModel *model;
 	gboolean saved = FALSE;
 
-	widget = glade_xml_get_widget (xml, combo);
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, combo));
 	g_assert (widget);
 	model = gtk_combo_box_get_model (GTK_COMBO_BOX (widget));
 	if (gtk_combo_box_get_active (GTK_COMBO_BOX (widget)) == PW_TYPE_SAVE) {
-		widget = glade_xml_get_widget (xml, entry);
+		widget = GTK_WIDGET (gtk_builder_get_object (builder, entry));
 		g_assert (widget);
 		password = gtk_entry_get_text (GTK_ENTRY (widget));
 		if (password && strlen (password)) {
@@ -727,34 +814,57 @@ save_secrets (NMVpnPluginUiWidgetInterface *iface,
 	VpncPluginUiWidget *self = VPNC_PLUGIN_UI_WIDGET (iface);
 	VpncPluginUiWidgetPrivate *priv = VPNC_PLUGIN_UI_WIDGET_GET_PRIVATE (self);
 	NMSettingConnection *s_con;
+	NMSettingVPN *s_vpn;
 	const char *id, *uuid;
+	NMSettingSecretFlags secret_flags = NM_SETTING_SECRET_FLAG_NONE;
 
 	s_con = (NMSettingConnection *) nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION);
-	if (!s_con) {
+	s_vpn = (NMSettingVPN *) nm_connection_get_setting (connection, NM_TYPE_SETTING_VPN);
+	if (!s_con || !s_vpn) {
 		g_set_error (error,
 		             VPNC_PLUGIN_UI_ERROR,
 		             VPNC_PLUGIN_UI_ERROR_INVALID_CONNECTION,
-		             "missing 'connection' setting");
+		             "missing connection or VPN settings");
 		return FALSE;
 	}
 
 	id = nm_setting_connection_get_id (s_con);
 	uuid = nm_setting_connection_get_uuid (s_con);
 
-	save_one_password (priv->xml, VPNC_USER_PASSWORD, uuid, id,
-	                   "user_password_entry", "user_pass_type_combo", "user password");
-	save_one_password (priv->xml, VPNC_GROUP_PASSWORD, uuid, id,
-	                   "group_password_entry", "group_pass_type_combo", "group password");
+	if (nm_setting_get_secret_flags (NM_SETTING (s_vpn), NM_VPNC_KEY_XAUTH_PASSWORD, &secret_flags, NULL)) {
+		if (secret_flags & NM_SETTING_SECRET_FLAG_AGENT_OWNED) {
+			save_one_password (priv->builder, VPNC_USER_PASSWORD, uuid, id,
+					           "user_password_entry", "user_pass_type_combo", "user password");
+		}
+	}
+
+	if (nm_setting_get_secret_flags (NM_SETTING (s_vpn), NM_VPNC_KEY_SECRET, &secret_flags, NULL)) {
+		if (secret_flags & NM_SETTING_SECRET_FLAG_AGENT_OWNED) {
+			save_one_password (priv->builder, VPNC_GROUP_PASSWORD, uuid, id,
+					           "group_password_entry", "group_pass_type_combo", "group password");
+		}
+	}
 
 	return TRUE;
 }
 
+static void
+is_new_func (const char *key, const char *value, gpointer user_data)
+{
+	gboolean *is_new = user_data;
+
+	/* If there are any VPN data items the connection isn't new */
+	*is_new = FALSE;
+}
+
 static NMVpnPluginUiWidgetInterface *
 nm_vpn_plugin_ui_widget_interface_new (NMConnection *connection, GError **error)
 {
 	NMVpnPluginUiWidgetInterface *object;
 	VpncPluginUiWidgetPrivate *priv;
-	char *glade_file;
+	char *ui_file;
+	NMSettingVPN *s_vpn;
+	gboolean new = TRUE;
 
 	if (error)
 		g_return_val_if_fail (*error == NULL, NULL);
@@ -767,18 +877,24 @@ nm_vpn_plugin_ui_widget_interface_new (NMConnection *connection, GError **error)
 
 	priv = VPNC_PLUGIN_UI_WIDGET_GET_PRIVATE (object);
 
-	glade_file = g_strdup_printf ("%s/%s", GLADEDIR, "nm-vpnc-dialog.glade");
-	priv->xml = glade_xml_new (glade_file, "vpnc-vbox", GETTEXT_PACKAGE);
-	if (priv->xml == NULL) {
+	ui_file = g_strdup_printf ("%s/%s", UIDIR, "nm-vpnc-dialog.ui");
+	priv->builder = gtk_builder_new ();
+
+	if (!gtk_builder_add_from_file (priv->builder, ui_file, error)) {
+		g_warning ("Couldn't load builder file: %s",
+		           error && *error ? (*error)->message : "(unknown)");
+		g_clear_error (error);
 		g_set_error (error, VPNC_PLUGIN_UI_ERROR, 0,
-		             "could not load required resources at %s", glade_file);
-		g_free (glade_file);
+		             "could not load required resources at %s", ui_file);
+		g_free (ui_file);
 		g_object_unref (object);
 		return NULL;
 	}
-	g_free (glade_file);
+	g_free (ui_file);
 
-	priv->widget = glade_xml_get_widget (priv->xml, "vpnc-vbox");
+	gtk_builder_set_translation_domain (priv->builder, GETTEXT_PACKAGE);
+
+	priv->widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "vpnc-vbox"));
 	if (!priv->widget) {
 		g_set_error (error, VPNC_PLUGIN_UI_ERROR, 0, "could not load UI widget");
 		g_object_unref (object);
@@ -786,6 +902,11 @@ nm_vpn_plugin_ui_widget_interface_new (NMConnection *connection, GError **error)
 	}
 	g_object_ref_sink (priv->widget);
 
+	s_vpn = nm_connection_get_setting_vpn (connection);
+	if (s_vpn)
+		nm_setting_vpn_foreach_data_item (s_vpn, is_new_func, &new);
+	priv->new_connection = new;
+
 	if (!init_plugin_ui (VPNC_PLUGIN_UI_WIDGET (object), connection, error)) {
 		g_object_unref (object);
 		return NULL;
@@ -806,8 +927,8 @@ dispose (GObject *object)
 	if (priv->widget)
 		g_object_unref (priv->widget);
 
-	if (priv->xml)
-		g_object_unref (priv->xml);
+	if (priv->builder)
+		g_object_unref (priv->builder);
 
 	G_OBJECT_CLASS (vpnc_plugin_ui_widget_parent_class)->dispose (object);
 }
@@ -1046,20 +1167,37 @@ import (NMVpnPluginUiInterface *iface, const char *path, GError **error)
 	if (pcf_file_lookup_string (pcf, "main", "UserName", &buf))
 		nm_setting_vpn_add_data_item (s_vpn, NM_VPNC_KEY_XAUTH_USER, buf);
 
-	if (pcf_file_lookup_string (pcf, "main", "UserPassword", &buf))
+	if (pcf_file_lookup_string (pcf, "main", "UserPassword", &buf)) {
 		nm_setting_vpn_add_secret (s_vpn, NM_VPNC_KEY_XAUTH_PASSWORD, buf);
+		nm_setting_set_secret_flags (NM_SETTING (s_vpn),
+		                             NM_VPNC_KEY_XAUTH_PASSWORD,
+		                             NM_SETTING_SECRET_FLAG_AGENT_OWNED,
+		                             NULL);
+	}
 
 	if (pcf_file_lookup_bool (pcf, "main", "SaveUserPassword", &bool_value)) {
+		NMSettingSecretFlags flags = NM_SETTING_SECRET_FLAG_AGENT_OWNED;
+
 		if (bool_value) {
 			nm_setting_vpn_add_data_item (s_vpn,
 			                              NM_VPNC_KEY_XAUTH_PASSWORD_TYPE,
 			                              NM_VPNC_PW_TYPE_SAVE);
-		}
+		} else
+			flags |= NM_SETTING_SECRET_FLAG_NOT_SAVED;
+
+		nm_setting_set_secret_flags (NM_SETTING (s_vpn),
+			                         NM_VPNC_KEY_XAUTH_PASSWORD,
+			                         flags,
+			                         NULL);
 	}
 
-	if (pcf_file_lookup_string (pcf, "main", "GroupPwd", &buf))
+	if (pcf_file_lookup_string (pcf, "main", "GroupPwd", &buf)) {
 		nm_setting_vpn_add_secret (s_vpn, NM_VPNC_KEY_SECRET, buf);
-	else {
+		nm_setting_set_secret_flags (NM_SETTING (s_vpn),
+		                             NM_VPNC_KEY_SECRET,
+		                             NM_SETTING_SECRET_FLAG_AGENT_OWNED,
+		                             NULL);
+	} else {
 		/* Handle encrypted passwords */
 		if (pcf_file_lookup_string (pcf, "main", "enc_GroupPwd", &buf)) {
 			char *decrypted;
@@ -1069,6 +1207,11 @@ import (NMVpnPluginUiInterface *iface, const char *path, GError **error)
 				nm_setting_vpn_add_secret (s_vpn, NM_VPNC_KEY_SECRET, decrypted);
 				memset (decrypted, 0, strlen (decrypted));
 				g_free (decrypted);
+
+				nm_setting_set_secret_flags (NM_SETTING (s_vpn),
+				                             NM_VPNC_KEY_SECRET,
+				                             NM_SETTING_SECRET_FLAG_AGENT_OWNED,
+				                             NULL);
 			}
 		}
 	}
@@ -1132,8 +1275,12 @@ import (NMVpnPluginUiInterface *iface, const char *path, GError **error)
 	}
 
 	if (pcf_file_lookup_string (pcf, "main", "DHGroup", &buf)) {
-		if (!strcmp (buf, "1") || !strcmp (buf, "2") || !strcmp (buf, "5"))
-			nm_setting_vpn_add_data_item (s_vpn, NM_VPNC_KEY_DHGROUP, buf);
+		if (!strcmp (buf, "1") || !strcmp (buf, "2") || !strcmp (buf, "5")) {
+			char *tmp;
+			tmp = g_strdup_printf ("dh%s", buf);
+			nm_setting_vpn_add_data_item (s_vpn, NM_VPNC_KEY_DHGROUP, tmp);
+			g_free (tmp);
+		}
 	}
 
 	if (pcf_file_lookup_string (pcf, "main", "X-NM-Routes", &buf))
@@ -1201,6 +1348,7 @@ export (NMVpnPluginUiInterface *iface,
 	gboolean save_password = FALSE;
 	gboolean use_natt = FALSE;
 	gboolean use_force_natt = FALSE;
+	NMSettingSecretFlags flags = NM_SETTING_SECRET_FLAG_NONE;
 
 	s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION));
 	s_ip4 = (NMSettingIP4Config *) nm_connection_get_setting (connection, NM_TYPE_SETTING_IP4_CONFIG);
@@ -1261,13 +1409,19 @@ export (NMVpnPluginUiInterface *iface,
 		peertimeout = value;
 
 	value = nm_setting_vpn_get_data_item (s_vpn, NM_VPNC_KEY_DHGROUP);
-	if (value && strlen (value))
-		dhgroup = value;
-
-	value = nm_setting_vpn_get_data_item (s_vpn, NM_VPNC_KEY_XAUTH_PASSWORD_TYPE);
 	if (value && strlen (value)) {
-		if (!strcmp (value, NM_VPNC_PW_TYPE_SAVE))
+		dhgroup = (value[0] == 'd' && value[1] == 'h') ? value + 2 : NULL;
+	}
+
+	if (nm_setting_get_secret_flags (NM_SETTING (s_vpn), NM_VPNC_KEY_XAUTH_PASSWORD, &flags, NULL)) {
+		if (!(flags & NM_SETTING_SECRET_FLAG_NOT_SAVED))
 			save_password = TRUE;
+	} else {
+		value = nm_setting_vpn_get_data_item (s_vpn, NM_VPNC_KEY_XAUTH_PASSWORD_TYPE);
+		if (value && strlen (value)) {
+			if (!strcmp (value, NM_VPNC_PW_TYPE_SAVE))
+				save_password = TRUE;
+		}
 	}
 
 	routes = g_string_new ("X-NM-Routes=");
